let first = [|
  [| -1; -1; -1; -1; -1; -1; -1; -1; -1; -1 |];
  [| -1; 0; 0; 0; 0; 0; 0; 0; 0; -1 |];
  [| -1; 0; 0; 0; 0; 0; 0; 0; 0; -1 |];
  [| -1; 0; 0; 0; 0; 0; 0; 0; 0; -1 |];
  [| -1; 0; 0; 0; 2; 1; 0; 0; 0; -1 |];
  [| -1; 0; 0; 0; 1; 2; 0; 0; 0; -1 |];
  [| -1; 0; 0; 0; 0; 0; 0; 0; 0; -1 |];
  [| -1; 0; 0; 0; 0; 0; 0; 0; 0; -1 |];
  [| -1; 0; 0; 0; 0; 0; 0; 0; 0; -1 |];
  [| -1; -1; -1; -1; -1; -1; -1; -1; -1; -1 |];
|]

let xdir = [| -1; -1; -1; 0; 0; 1; 1; 1 |]
let ydir = [| -1; 0; 1; -1; 1; -1; 0; 1 |]

let boards =
  Array.init 62 (fun i -> (Array.init 10 (fun x -> Array.init 10 (fun y -> (first.(x).(y))))))

let board t =
  boards.(t)

let each t f =
  let b = board t in
  let rec run_y y =
    let rec run_x x =
      if (x <= 8) then
        let v = b.(x).(y) in
          f x y v;
          run_x (x+1)
    in
      if (y <= 8) then (
        run_x 1;
        run_y (y+1)
      )
  in
    run_y 1

let accum t f a =
  let b = board t in
  let rec run_y y a =
    let rec run_x x a =
      if (x <= 8) then
        let v = b.(x).(y) in
        let n = f x y v a in
          run_x (x+1) n
      else
        a
    in
      if (y <= 8) then (
        let n = run_x 1 a in
          run_y (y+1) n
      )
      else
        a
  in
    run_y 1 a

let can_put t c x y =
  let b = (board t) in
  let v = b.(x).(y) in
    if v != 0 then false
    else
      let e = (3 - c) in
      let rec can_put_dir i =
        if (i == 8) then
          false
        else
          let xn = x + xdir.(i) in
          let yn = y + ydir.(i) in
            if (e != b.(xn).(yn)) then
              can_put_dir (i+1)
            else
              let rec end_color x y =
                let v = b.(x).(y) in
                  if (v == e) then
                    end_color (x + xdir.(i)) (y + ydir.(i))
                  else
                    v
              in
                if (end_color (xn + xdir.(i)) (yn + ydir.(i)) == c) then
                  true
                else
                  can_put_dir (i+1)
      in
        can_put_dir 0

let can_put_all t c =
  let rec run_y y =
    let rec run_x x =
      if (x > 8) then
        false
      else
        if (can_put t c x y) then
          true
        else
          run_x (x+1)
    in
      if (y > 8) then
        false
      else
        if (run_x 1) then
          true
        else
          run_y (y+1)
  in
    run_y 1

let copy t nt =
  let n = (board nt) in
    each t (fun x y v -> n.(x).(y) <- v)

let put t c x y =
  copy t (t+1);
  let b = (board (t+1)) in
  let v = b.(x).(y) in
    if v != 0 then false
    else
      let e = (3 - c) in
      let rec can_put_dir i =
        if (i == 8) then
          false
        else
          let xd = xdir.(i) in
          let yd = ydir.(i) in
          let xn = x + xd in
          let yn = y + yd in
            if (e != b.(xn).(yn)) then
              can_put_dir (i+1)
            else
              let rec end_color x y =
                let v = b.(x).(y) in
                  if (v == e) then
                    end_color (x + xd) (y + yd)
                  else
                    v
              in
                if (end_color (xn + xd) (yn + yd) == c) then
                  let rec run_put x y =
                    let v = b.(x).(y) in
                      if (v != c) then (
                        b.(x).(y) <- c;
                        run_put (x + xd) (y + yd)
                      )
                  in
                    b.(x).(y) <- c;
                    run_put (x + xd) (y + yd);
                    can_put_dir (i+1)
                else
                  can_put_dir (i+1)
      in
        can_put_dir 0

let num t c =
  accum t (fun x y v a -> if (v == c) then a+1 else a) 0

let num_diff t c =
  let e = (3 - c) in
    accum t (
      fun x y v a ->
        if (v == c) then a+1
        else
          if (v == e) then a-1
          else a
    ) 0

let can_put_num t c =
  accum t (fun x y v a -> if (can_put t c x y) then a+1 else a) 0

let can_put_num_diff t c =
  let e = (3 - c) in
    accum t (fun x y v a ->
               if (can_put t c x y) then
                 if (can_put t e x y) then a else a+1
               else
                 if (can_put t e x y) then a-1 else a
            ) 0

let dump t =
  let stones = [| ""; ""; "" |] in
  let nums = [| "dummy";  ""; ""; ""; ""; ""; ""; ""; "" |] in
    print_string "\n";
    each t (
      fun x y v ->
        if (x == 1) then Printf.printf "\n%s" nums.(y);
        Printf.printf "%s" stones.(v)
    );
    print_string "\n\n";
    flush stdout

let info t =
  dump t;
  Printf.printf "%d %d\n" (num t 1) (num t 2);
  print_string "\n\n"

