let connect host port =
  let addr = Unix.gethostbyname host in
  let s =
    Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
    Unix.connect s (Unix.ADDR_INET(addr.Unix.h_addr_list.(0), port)); s

let send socks msg =
  let (isock, osock) = socks in
    print_string ("! " ^ msg ^ "\n");
    output_string osock (msg ^ "\n");
    flush stdout;
    flush osock

let pong socks serv =
  send socks ("PONG " ^ serv)

let notice socks msg =
  send socks ("NOTICE " ^ Conf.channel ^ " :" ^ msg)

let privmsg socks msg =
  send socks ("PRIVMSG " ^ Conf.channel ^ " :" ^ msg)

let wait_notice socks =
  let (isock, osock) = socks in
  let ping_reg = Str.regexp "PING \\(.*\\)" in
  let notice_reg = Str.regexp "[^ ]* NOTICE [^ ]* :\\(.*\\)" in
  let rec wait_start () =
    let line = input_line isock in
      print_string (line ^ "\n"); flush stdout;
      if (Str.string_match ping_reg line 0) then (
        pong socks (Str.matched_group 1 line);
        wait_start ()
      )
      else
        if (Str.string_match notice_reg line 0) then
          Str.matched_group 1 line
        else
          wait_start ()
  in
    wait_start ()

let wait_privmsg socks =
  let (isock, osock) = socks in
  let ping_reg = Str.regexp "PING \\(.*\\)" in
  let privmsg_reg = Str.regexp "[^ ]* PRIVMSG [^ ]* :\\(.*\\)" in
  let rec wait_start () =
    let line = input_line isock in
      print_string (line ^ "\n"); flush stdout;
      if (Str.string_match ping_reg line 0) then (
        pong socks (Str.matched_group 1 line);
        wait_start ()
      )
      else
        if (Str.string_match privmsg_reg line 0) then
          Str.matched_group 1 line
        else
          wait_start ()
  in
    wait_start ()

let init() =
  let sock = connect Conf.host Conf.port in
  let isock = Unix.in_channel_of_descr sock in
  let osock = Unix.out_channel_of_descr sock in
  let socks = (isock, osock) in
    send socks ("NICK " ^ Conf.nick);
    send socks ("USER " ^ Conf.user ^ " 0 * " ^ Conf.real);
    send socks ("JOIN " ^ Conf.channel);
    privmsg socks "@soloB";

    let wait_reg = Str.regexp "tello_start" in
    let rec wait_start () =
      let msg = wait_notice (isock, osock) in
        if (not (Str.string_match wait_reg msg 0)) then
          wait_start ()
    in
      wait_start ();
      (isock, osock)

let think socks t c =
  let wb = if (c == 1) then "B" else "W" in
  let wait_reg = Str.regexp (wb ^ ">\\([1-8]\\),\\([1-8]\\)") in
  let rec wait_start () =
    let msg = wait_notice socks in
      if (Str.string_match wait_reg msg 0) then
        let x = int_of_string (Str.matched_group 1 msg) in
        let y = int_of_string (Str.matched_group 2 msg) in
          if (Board.can_put t c x y) then
            (x, y)
          else (
            notice socks "you cannot put here, i think.";
            wait_start ()
          )
      else
        wait_start ()
  in
    wait_start ()

let think_sugar_bot socks t c =
  let wb = if (c == 1) then "B" else "W" in
  let wait_reg = Str.regexp ("@" ^ wb ^ "\\([A-H]\\)\\([0-7]\\)") in
  let rec wait_start () =
    let msg = wait_privmsg socks in
      if (Str.string_match wait_reg msg 0) then
        let x = ((int_of_char (Str.matched_group 1 msg).[0]) - (int_of_char 'A') + 1) in
        let y = (int_of_string (Str.matched_group 2 msg) + 1) in
          if (Board.can_put t c x y) then
            (x, y)
          else (
            notice socks "you cannot put here, i think.";
            wait_start ()
          )
      else
        wait_start ()
  in
    wait_start ()

let think_sugar_ai socks t c =
  let wb = if (c == 1) then "B" else "W" in
  let wait_reg = Str.regexp ("+AI " ^ wb ^ "\\([A-H]\\)\\([0-7]\\)") in
  let rec wait_start () =
    let msg = wait_privmsg socks in
      if (Str.string_match wait_reg msg 0) then
        let x = ((int_of_char (Str.matched_group 1 msg).[0]) - (int_of_char 'A') + 1) in
        let y = (int_of_string (Str.matched_group 2 msg) + 1) in
          if (Board.can_put t c x y) then
            (x, y)
          else (
            notice socks "you cannot put here, i think.";
            wait_start ()
          )
      else
        wait_start ()
  in
    wait_start ()

let report socks c x y =
  let wb = if (c == 1) then "B" else "W" in
    notice socks (wb ^ ">" ^ (string_of_int x) ^ "," ^ (string_of_int y))

let report_sugar_bot socks c x y =
  let ah = [| "A"; "B"; "C"; "D"; "E"; "F"; "G"; "H" |] in
  let wb = if (c == 1) then "B" else "W" in
    privmsg socks ("@" ^ wb ^ ah.(x-1) ^ (string_of_int (y-1)))

